import {
  LoggerConfig,
  LoggerLevel,
} from '../core';


export abstract class LoggerService {

  public constructor(
    private readonly config: LoggerConfig
  ) {

  }

  public trace(message, ...additional: any[]): void {
    this._log(LoggerLevel.TRACE, message, additional);
  }

  public debug(message, ...additional: any[]): void {
    this._log(LoggerLevel.DEBUG, message, additional);
  }

  public info(message, ...additional: any[]): void {
    this._log(LoggerLevel.INFO, message, additional);
  }

  public log(message, ...additional: any[]): void {
    this._log(LoggerLevel.LOG, message, additional);
  }

  public warn(message, ...additional: any[]): void {
    this._log(LoggerLevel.WARN, message, additional);
  }

  public error(message, ...additional: any[]): void {
    this._log(LoggerLevel.ERROR, message, additional);
  }

  private _timestamp(): string {
    return new Date().toISOString();
  }

  private _log(level: LoggerLevel, logOnServer: boolean, message, additional: any[] = []): void {
    if (!message) {
      return;
    }

    // Allow logging on server even if client log level is off
    // if (logOnServer) {
    //   this._logOnServer(level, message, additional);
    // }

    // if no message or the log level is less than the environ
    if (level < this.config.level) {
      return;
    }

    try {
      message = typeof message === 'string' ? message
        : JSON.stringify(message, null, 2);
    } catch (e) {
      additional = [message, ...additional];
      message = 'The provided "message" value could not be parsed with JSON.stringify().';
    }

    // Coloring doesn't work in IE
    // if (this._isIE) {
    //   return this._logIE(level, message, additional);
    // }

    // const color = this._getColor(level);

    // console.log(`%c${this._timestamp()} [${Levels[level]}]`, `color:${color}`, message, ...additional);
  }

  private _getColor(level: LoggerLevel): 'blue' | 'teal' | 'gray' | 'red' | undefined {
    switch (level) {
      case LoggerLevel.TRACE:
        return 'blue';
      case LoggerLevel.DEBUG:
        return 'teal';
      case LoggerLevel.INFO:
      case LoggerLevel.LOG:
        return 'gray';
      case LoggerLevel.WARN:
      case LoggerLevel.ERROR:
        return 'red';
      case LoggerLevel.OFF:
      default:
        return;
    }
  }
}
