import { Component, OnInit, OnChanges, Input, Output, EventEmitter, ViewChild, AfterViewInit } from '@angular/core';
import { Router } from '@angular/router';
import { Store, select } from '@ngrx/store';
import { RPCError } from 'packages/core/rpc/error';

import * as ListStore from 'packages/meta/sensor-display-item/store/list';
import { ReadAllSensorDisplayItemByCrawlerSelector } from 'packages/meta/sensor-display-item/store';
import { MetaSensorDisplayItem } from 'packages/meta/sensor-display-item/model/MetaSensorDisplayItem';
import { MetaCrawler } from '../../../../meta/crawler/model/MetaCrawler';
import { ITreeOptions } from 'angular-tree-component';

@Component({
  selector: 'of-sensor-item-selector',
  templateUrl: './sensor-item-selector.component.html',
  styleUrls: ['./sensor-item-selector.component.scss']
})
export class SensorItemSelectorComponent implements OnInit, AfterViewInit, OnChanges {

  @Input() selectedCrawler: MetaCrawler;
  @Output() selectedItems = new EventEmitter<MetaSensorDisplayItem[]>();
  items$ = this.listStore.pipe(select(ReadAllSensorDisplayItemByCrawlerSelector.select('list')));
  sensorDisplayItems: MetaSensorDisplayItem[];
  @ViewChild('tree') tree;
  nodes;

  constructor(
    private router: Router,
    private listStore: Store<ListStore.State>,
  ) { }

  ngOnInit() {
    this.items$.subscribe(
      (list: MetaSensorDisplayItem[]) => {
        if (list !== null) {
          console.log('###Display Items###');
          console.log(list);
          console.log('###################');
          this.sensorDisplayItems = list;
          this.generateTreeNode(list);
        }
      },
      (error: RPCError) => {
        console.log(error.response.message);
      }
    );
  }

  ngAfterViewInit() {
  }

  ngOnChanges() {
    this.listStore.dispatch(new ListStore.ReadAllByCrawler(this.selectedCrawler));
  }

  generateTreeNode(list: MetaSensorDisplayItem[]) {
    this.nodes = new Array();
    for (const item of list) {
      const categoryNode = this.existCategory(item);
      if (categoryNode === null) {
        const childrenNode = new Array();
        childrenNode.push(this.getChildNode(item));
        const node = {
          title: item.itemType.name,
          children: childrenNode,
          isExpanded: true
        };
        this.nodes.push(node);
      } else {
        categoryNode.children.push(this.getChildNode(item));
      }
    }
  }

  getChildNode(item) {
    const childNode = {
      title: item.displayName,
      key: ' - ' + item.key,
      default: item.default
    };
    return childNode;
  }

  existCategory(item: MetaSensorDisplayItem) {
    let categoryNode = null;
    for (const node of this.nodes) {
      if (node.title === item.itemType.name) {
        categoryNode = node;
      }
    }
    return categoryNode;
  }

  checkItem(node) {
    node.toggleActivated(true);
    console.log(node);
    // if (node.isActive) {
    //   if (this.checkedSet.has(node.data) === false) {
    //     this.checkedSet.add(node.data);
    //   }
    // } else {
    //   if (this.checkedSet.has(node.data)) {
    //     this.checkedSet.delete(node.data);
    //   }
    // }
  }

  onTreeDataLoad(tree) {
    tree.treeModel.expandAll();
  }
}
